      subroutine aptchit (nchar, ngrp, ngrpt, nwtot, nerr)

ccbeg.
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c                             SUBROUTINE APTCHIT
c
c     call aptchit (nchar, ncom, nerr)
c
c     Version:  aptchit  Updated    2007 January 26 16:40.
c               aptchit  Originated 1991 November 21 18:00.
c
c     Author:   Arthur L. Edwards, LLNL, L-298, Telephone (925) 422-4123.
c
c
c     Purpose:  To find ncom, the number of ways the nchar letters in character
c               string , starting at character number , can be
c               rearraged into ngroup groups, with the order of the letters in
c               each group the same as in the original string, but without any
c               intermingled letters from the other groups.
c
c     Example:  The letters in the STRING "ganeamew" can be rearranged into the
c               groups "a", "new", "game", with the initial letter positions
c               indicated in upper case:  "gAneamew", "gaNEameW", "GaneAMEw.
c
c     Input:    , , nchar, ngroup.
c
c     Output:   ncom, nerr.
c
c     Glossary:
c
c          Input    An array containing a  of length nchar,
c                          starting at character number .
c
c          Input    The character position in array  of the first
c                          character of the .  E. g.,  = 1
c                          means the leftmost character of (1).
c                          Must be positive.
c
c     nchar     Input    The length of the . Must be positive.
c
c     ncom      Output   The number of ways the  can be
c                          rearranged into ngroup groups, with the characters
c                          in each group in the same order as in the character
c                          string.
c
c     nerr      Output   Indicates an input or result error, if not zero.
c                          1 if nchar is not positive.
c
c     ngroup    Input    The number of groups into which the 
c                          may be rearranged.
c
c     Changes:  2007 January 26 16:40.  Originated.
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccend.

c.... Arguments.

      character *1 (1)           ! Array containing . (1
      integer                    ! Position in  of first character.
      integer   nchar                 ! The number of characters to rearrange.
      integer   ncom                  ! The number of rearrangements possible.
      integer   nerr                  ! Error flag.
      integer   ngroup                ! The number of groups allowed.

c.... Local variables.

      integer n                       ! Index in a  or group.

c*** DEBUG begins.
 9901 format (/ 'aptchit counting ways to rearrange' /
     &  '  a group of ',i5,' characters into ',i4,' groups.' )
 9903 format ('  (',i3,') = ',256a1)
      write ( 3, 9901) nchar, ngroup
      nmax =  + nchar - 1
      if (( .gt. 0) .and. (nchar .gt. 0)) then
        write ( 3, 9903) , ((n), n = , nmax)
      endif

c*** DEBUG ends.

c.... initialize.

      nerr = 0

c.... Test for input errors.

      if (nchar .le. 0) then
        nerr = 1
        go to 210
      endif

      if ( .le. 0) then
        nerr = 2
        go to 210
      endif

c.... Find number of possible arrangements.
      
      nlet = ngroup
      
      do n = 1, nlet                 ! Loop over puzzle word lengths.
        nan(n,1)  = 1
        nan(n,n)  = 1
      enddo                          ! End of loop over puzzle word lengths.
      
      nletm = nlet - 1
      do n = 1, nletm                ! Loop over puzzle word lengths.
        np = n + 1
        do nw = np, nlet             ! Loop over impossible numbers of groups.
          nan(n,nw) = 0
        enddo                        ! End of loop over impossible numbers.
      enddo                          ! End of loop over string lengths.
      
      ntot(1)  = 1
      ntotw(1) = 1
      do n = 2, nlet                 ! Loop over string lengths.
        ntot(n)  = 1
        ntotw(n) = 1
        do nw = 2, n                 ! Loop over number of groups.
          nm        = n - 1
          nwm       = nw - 1
          nadd      = nw       * nan(nm,nw)
          nan(n,nw) = nadd     + nan(nm,nwm)
          ntot(n)   = ntot(n)  + nan(n,nw)
          nwadd     = nw       * nan(n,nw)
          ntotw(n)  = ntotw(n) + nwadd
        enddo                        ! End of loop over number of groups.
      enddo                          ! End of loop over string lengths.
      
      do n = 1, nlet                 ! Loop over string lengths.
        fntotw    = ntotw(n)
        fntot     = ntot(n)
        fnwavg(n) = fntotw / fntot
      enddo                          ! End of loop over string lengths.
      
  210 continue
c*** DEBUG begins.
 9908 format (/ 'aptchit results:  nerr=',i2,'.' /
     &  '  n           ncom' )
 9909 format (i3,i15)
      write ( 3, 9908) nerr
      write ( 3, 9909) (n, ncom(n), n = 1, ngroup)
c*** DEBUG ends.
      return

c.... End of subroutine aptchit.      (+1 line.)
      end
