      subroutine aptconm (ind, np, pgcon, nerr)

ccbeg.
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c                             SUBROUTINE APTCONM
c
c     call aptconm (ind, np, pgcon, nerr)
c
c     Version:  aptconm  Updated    2006 September 1 15:00.
c               aptconm  Originated 1989 March 2 15:30.
c
c     Author:   Arthur L. Edwards, LLNL, L-298, Telephone (925) 422-4123.
c
c     Refs:     1.  Peter J. Mohr and Barry M. Taylor, CODATA Recommended Values
c               of the Fundamental Physical Constants: 1998, Journal of
c               Physical and Chemical Reference Data, Vol 28, No. 6, 1999,
c               and Reviews of Modern Physics, Vol. 72, No. 2, 2000.
c
c               2.  "Metric Practice Guide E 380-72e", 1973, American
c               Society for Testing and Materials, 1916 Race Street,
c               Philadelphia, Pa 19103.
c
c               3.  Charles Seife, "A Slow Carousel Ride Gauges Gravity's
c               Pull', Science Vol 288, 2000 May 12.
c
c               4.  Peter J. Mohr and Barry N. Taylor, CODATA Recommended
c               Values of the Fundamental Physical Constants: 2002, to be
c               published:  <http://physics.nist.gov/constants>.
c               NOT YET IMPLEMENTED IN THIS SUBROUTINE.
c
c
c     Purpose:  To provide the best available values of the fundamental
c               physical constants and conversion factors of physics and
c               chemistry, in SI units.  See aptconl for cm-g-shake-keV units.
c               For each of np values of ind, the constant with local index ind
c               (1-217) is returned in array pgcon.  The local indices are
c               tabulated below.  For definitions of the constants, see the
c               full listing of this subroutine, or reference 3 listed above.
c               If any value of ind(n) is not in the range from 1 to 217, the
c               returned value of pgcon(n) will be -1.e+99.
c               Flag nerr indicates any input error, if not zero.
c
c             1 pga        45 pgevtokg   89 pgjtokg   133 pgmpev    177 pgratea
c             2 pgabi85    46 pgevtorm   90 pgjtorm   134 pgmpj     178 pgrated
c             3 pgad       47 pgfar      91 pgk       135 pgmsubd   179 pgratemu
c             4 pgalpha    48 pgg        92 pgkd      136 pgmsube   180 pgratep
c             5 pgalphai   49 pggamma    93 pgkgtoam  137 pgmsubmu  181 pgratmue
c             6 pgamtoev   50 pggamp     94 pgkgtoev  138 pgmsubn   182 pgratne
c             7 pgamtoht   51 pggampb    95 pgkgtoht  139 pgmsubp   183 pgratnp
c             8 pgamtohz   52 pggampp    96 pgkgtohz  140 pgmub     184 pgratpe
c             9 pgamtoj    53 pggamppb   97 pgkgtoj   141 pgmube    185 pgratpmu
c            10 pgamtok    54 pggolden   98 pgkgtok   142 pgmubf    186 pgrmold
c            11 pgamtokg   55 pggsube    99 pgkgtorm  143 pgmubk    187 pgrmole
c            12 pgamtorm   56 pggsubmu  100 pgktoam   144 pgmubw    188 pgrmtoam
c            13 pgamu      57 pggsubn   101 pgktoev   145 pgmud     189 pgrmtoev
c            14 pgamud     58 pgh       102 pgktoht   146 pgmudb    190 pgrmtoht
c            15 pgamue     59 pghart    103 pgktohz   147 pgmudn    191 pgrmtohz
c            16 pgamumu    60 pghartev  104 pgktoj    148 pgmue     192 pgrmtoj
c            17 pgamun     61 pghbar    105 pgktokg   149 pgmueb    193 pgrmtok
c            18 pgamup     62 pghbarc   106 pgktorm   150 pgmuen    194 pgrmtokg
c            19 pgastar    63 pghbarok  107 pglamce   151 pgmumu    195 pgrydb
c            20 pgasube    64 pghlen    108 pglamceb  152 pgmumub   196 pgrydbc
c            21 pgasubmu   65 pghmass   109 pglamcn   153 pgmumun   197 pgrydbe
c            22 pgatm      66 pghok     110 pglamcnb  154 pgmun     198 pgrydbev
c            23 pgatomx    67 pghtime   111 pglamcp   155 pgmune    199 pgsate
c            24 pgbohr     68 pghttoam  112 pglamcpb  156 pgmunf    200 pgsatez
c            25 pgc        69 pghttoev  113 pglosch   157 pgmunk    201 pgsb
c            26 pgcsub1    70 pghttohz  114 pgmagfq   158 pgmunt    202 pgsbd
c            27 pgcsub2    71 pghttoj   115 pgmdev    159 pgmuntb   203 pgsia
c            28 pgcsub2d   72 pghttok   116 pgmdj     160 pgmuntn   204 pgsid220
c            29 pgcunit    73 pghttokg  117 pgmeev    161 pgmunw    205 pgsigw
c            30 pgcuxun    74 pghttorm  118 pgmej     162 pgmup     206 pgsivolm
c            31 pgdrohm    75 pghyfsph  119 pgmmrde   163 pgmupb    207 pgtemp
c            32 pge        76 pghztoam  120 pgmmrdp   164 pgmupn    208 pgtempt
c            33 pgebase    77 pghztoev  121 pgmmremu  165 pgmupp    209 pgthomx
c            34 pgec       78 pghztoht  122 pgmmrep   166 pgmuppb   210 pgubb
c            35 pgeoh      79 pghztoj   123 pgmmrmup  167 pgmuppn   211 pgubbd
c            36 pgeome     80 pghztok   124 pgmmrne   168 pgmuvac   212 pgv76bi
c            37 pgeomp     81 pghztokg  125 pgmmrnp   169 pgnsuba   213 pgvmolt
c            38 pgepsvac   82 pghztorm  126 pgmmuev   170 pgombi85  214 pgvmoltz
c            39 pgerad     83 pgjfvr    127 pgmmuj    171 pgqcirc   215 pgwien
c            40 pgevtoam   84 pgjtoamu  128 pgmnev    172 pgqcirch  216 pgwiend
c            41 pgevtoht   85 pgjtoev   129 pgmnj     173 pgqhc     217 pi
c            42 pgevtohz   86 pgjtoht   130 pgmolh    174 pgqhr
c            43 pgevtoj    87 pgjtohz   131 pgmolhc   175 pgratde
c            44 pgevtok    88 pgjtok    132 pgmoxun   176 pgratdp
c
c     Input:    ind, np.
c
c     Output:   pgcon, nerr.
c
c     Glossary:
c
c     ind       Input    Indices of constants to be stored in array pgcon.
c                          If ind(n)<1 or ind(n)>217, -1.e99 will be returned
c                          for pgcon(n).  Size np.
c
c     nerr      Output   Error flag.  0 if no errors.
c                        1 if np is not positive.
c
c     np        Input    Size of arrays ind, pgcon.
c
c     pgcon     Output   Constants.  Value pgcon(n) is the value of the constant
c                          with index ind(n), if ind(n)>0 and ind(n)<218.
c                          Otherwise, pgcon(n) will be -1.e99.  Size np.
c
c     Definitions:
c
c               SI UNIT AND CM-G-SHAKE UNIT CONVERSION FACTORS
c
c                   Definitions and conversion of base units
c
c     Length or distance.  1 cm (centimeter) = 0.01 m (meter).
c
c     Mass.  1 g (gram) = 0.001 kg (kilogram).
c
c     Time.  1 sh (shake) = 1.0e-08 s (second).
c
c     Electric current.  1 labA = 1 (jerk / cm)**0.5 = 1.0e+09 A (ampere).
c                        1 A = 1 C / s = (1.0e-07 J / m)**0.5.
c
c     Temperature.  1 keV = 1.1604506e+07 K (Kelvin).
c                   1 K = 1 / 273.16 of the thermodynamic temperature of
c                   the triple point of water.
c     Note:  1988 NBS triple point of Gallium = 302.9169 K (1.7 ppm).
c
c                Definitions and conversion of derived units
c
c     Acceleration.  1 cm / sh**2 = 1.0e+14 m / s**2.
c
c     Area.  1 cm**2 = 1.0e-04 m**2.
c
c     Density.  1 g / cm**3 = 1.0e+03 kg / m**3.
c
c     Electric capacitance.  1 labF = 1 sh**2 / cm = 1.0e-07 F (farad).
c                            1 F = 1 A s / V = 1.0e-07 s**2 / m.
c
c     Electric inductance. 1 labH = 1 cm = 1.0e-09 H (henry).
c                          1 H = 1 V s / A = 1.0e+07 m.
c
c     Electric resistance.  1 labohm = 1 cm / sh = 0.1 ohm.
c                           1 ohm = 1 V / A = 1.0e+07 m / s.
c
c     Electric voltage.  1 labV = 1 (jerk cm / sh**2)**0.5 = 1.0e+08 V (volt).
c                        1 V = 1 W / A = (1.0e+07 J m / s**2)**0.5.
c
c     Electrical charge.  1 labC = 1 (jerk sh**2 / cm)**0.5 = 10 C (coulomb).
c                         1 C = 1 A s = (1.0e-07 J s**2 / m)**0.5.
c
c     Electrical conductance.  1 labS = 1 sh / cm = 10 S (siemen).
c                              1 S = 1 A / V = 1.0e-07 s / m.
c
c     Electron volts.  1 keV (kiloelectron volts) = 1000 eV (electron volts).
c                      1 eV = 1.602176462e-19 J.
c
c     Energy or work.  1 jerk = 1.0e+09 J (joule).
c                      1 J = 1 kg m**2 / s**2 = 1 N m (newton meter).
c
c     Force.  1 g cm / sh**2 = 1 jerk / cm = 1.0e+07 N (newton).
c             1 N = 1 Kg m / s**2 = 1 J / m.
c
c     Magnetic flux.  1 labWb = 1 (jerk cm)**0.5 = 1 Wb (weber).
c                     1 Wb = 1 V s = (1.0e+07 J m)**0.5.
c
c     Magnetic flux density.  1 labT = 1 (jerk / cm**3)**0.5 = 10,000 T (tesla).
c                             1 T = 1 Wb / m**2 = (1.0e+07 J / m**3)**0.5.
c
c     Power.  1 jerk / sh = 1.0e+17 W (watt)
c             1 W = 1 J / s = 1.0e-17 jerk / sh.
c
c     Pressure or energy density.  1 jerk / cm**3 = 1.0e+15 Pa (pascal).
c                                  1 Pa = 1 N / m**2 = 1 J / m**3.
c
c     Specific heat.  1 jerk / (g keV) = 8.617342e+04 J / (kg K).
c
c     Thermal conductivity.  1 jerk / (sh cm keV) = 8.617342e+11 W / (m K).
c
c     Velocity.  1 cm / sh = 1.0e+06 m / s.
c
c     Volume.  1 cm**3 = 1.0e-06 m**3.
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccend.

c===============================================================================
c
c File:  physcon.2002  Version 2005 January 14 14:20.
c 
c              Fundamental Physical Constants --- Complete Listing
c 
c 
c   From:  http://physics.nist.gov/constants
c 
c Source: Peter J. Mohr and Barry N. Taylor, CODATA Recommended Values of the 
c         Fundamental Physical Constants: 2002,
c         To be published.
c 
c 
c   Quantity                                               Value                 Uncertainty          Unit
c ------------------------------------------------------------------------------------------------------------------------
c speed of light in vacuum                               299 792 458           (exact)               m s^-1
c magn. constant                                         12.566 370 614...e-7  (exact)               N A^-2
c electric constant                                      8.854 187 817...e-12  (exact)               F m^-1
c characteristic impedance of vacuum                     376.730 313 461...    (exact)               ohm
c Newtonian constant of gravitation                      6.6742e-11            0.0010e-11            m^3 kg^-1 s^-2
c Newtonian constant of gravitation over h-bar c         6.7087e-39            0.0010e-39            (GeV/c^2)^-2
c Planck constant                                        6.626 0693e-34        0.000 0011e-34        J s
c Planck constant in eV s                                4.135 667 43e-15      0.000 000 35e-15      eV s
c Planck constant over 2 pi times c in MeV fm            197.326 968           0.000 017             MeV fm
c Planck constant over 2 pi                              1.054 571 68e-34      0.000 000 18e-34      J s
c Planck constant over 2 pi in eV s                      6.582 119 15e-16      0.000 000 56e-16      eV s
c Planck mass                                            2.176 45e-8           0.000 16e-8           kg
c Planck temperature                                     1.416 79e32           0.000 11e32           K
c Planck length                                          1.616 24e-35          0.000 12e-35          m
c Planck time                                            5.391 21e-44          0.000 40e-44          s
c elementary charge                                      1.602 176 53e-19      0.000 000 14e-19      C
c elementary charge over h                               2.417 989 40e14       0.000 000 21e14       A J^-1
c magn. flux quantum                                     2.067 833 72e-15      0.000 000 18e-15      Wb
c conductance quantum                                    7.748 091 733e-5      0.000 000 026e-5      S
c inverse of conductance quantum                         12 906.403 725        0.000 043             ohm
c Josephson constant                                     483 597.879e9         0.041e9               Hz V^-1
c von Klitzing constant                                  25 812.807 449        0.000 086             ohm
c Bohr magneton                                          927.400 949e-26       0.000 080e-26         J T^-1
c Bohr magneton in eV/T                                  5.788 381 804e-5      0.000 000 039e-5      eV T^-1
c Bohr magneton in Hz/T                                  13.996 2458e9         0.000 0012e9          Hz T^-1
c Bohr magneton in inverse meters per tesla              46.686 4507           0.000 0040            m^-1 T^-1
c Bohr magneton in K/T                                   0.671 7131            0.000 0012            K T^-1
c nuclear magneton                                       5.050 783 43e-27      0.000 000 43e-27      J T^-1
c nuclear magneton in eV/T                               3.152 451 259e-8      0.000 000 021e-8      eV T^-1
c nuclear magneton in MHz/T                              7.622 593 71          0.000 000 65          MHz T^-1
c nuclear magneton in inverse meters per tesla           2.542 623 58e-2       0.000 000 22e-2       m^-1 T^-1
c nuclear magneton in K/T                                3.658 2637e-4         0.000 0064e-4         K T^-1
c fine-structure constant                                7.297 352 568e-3      0.000 000 024e-3      
c inverse fine-structure constant                        137.035 999 710       0.000 000 096        
c Rydberg constant                                       10 973 731.568 525    0.000 073             m^-1
c Rydberg constant times c in Hz                         3.289 841 960 360e15  0.000 000 000 022e15  Hz
c Rydberg constant times hc in J                         2.179 872 09e-18      0.000 000 37e-18      J
c Rydberg constant times hc in eV                        13.605 6923           0.000 0012            eV
c Bohr radius                                            0.529 177 2108e-10    0.000 000 0018e-10    m
c Hartree energy                                         4.359 744 17e-18      0.000 000 75e-18      J
c Hartree energy in eV                                   27.211 3845           0.000 0023            eV
c quantum of circulation                                 3.636 947 550e-4      0.000 000 024e-4      m^2 s^-1
c quantum of circulation times 2                         7.273 895 101e-4      0.000 000 048e-4      m^2 s^-1
c Fermi coupling constant                                1.166 39e-5           0.000 01e-5           GeV^-2
c weak mixing angle                                      0.222 15              0.000 76              
c electron mass                                          9.109 3826e-31        0.000 0016e-31        kg
c electron mass in u                                     5.485 799 0945e-4     0.000 000 0024e-4     u
c electron mass energy equivalent                        8.187 1047e-14        0.000 0014e-14        J
c electron mass energy equivalent in MeV                 0.510 998 918         0.000 000 044         MeV
c electron-muon mass ratio                               4.836 331 67e-3       0.000 000 13e-3       
c electron-tau mass ratio                                2.875 64e-4           0.000 47e-4           
c electron-proton mass ratio                             5.446 170 2173e-4     0.000 000 0025e-4     
c electron-neutron mass ratio                            5.438 673 4481e-4     0.000 000 0038e-4     
c electron-deuteron mass ratio                           2.724 437 1095e-4     0.000 000 0013e-4     
c electron to alpha particle mass ratio                  1.370 933 555 75e-4   0.000 000 000 61e-4   
c electron charge to mass quotient                       -1.758 820 12e11      0.000 000 15e11       C kg^-1
c electron molar mass                                    5.485 799 0945e-7     0.000 000 0024e-7     kg mol^-1
c Compton wavelength                                     2.426 310 238e-12     0.000 000 016e-12     m
c Compton wavelength over 2 pi                           386.159 2678e-15      0.000 0026e-15        m
c classical electron radius                              2.817 940 325e-15     0.000 000 028e-15     m
c Thomson cross section                                  0.665 245 873e-28     0.000 000 013e-28     m^2
c electron magn. moment                                  -928.476 412e-26      0.000 080e-26         J T^-1
c electron magn. moment to Bohr magneton ratio           -1.001 159 652 1859   0.000 000 000 0038    
c electron magn. moment to nuclear magneton ratio        -1838.281 971 07      0.000 000 85          
c electron magn. moment anomaly                          1.159 652 1859e-3     0.000 000 0038e-3     
c electron g factor                                      -2.002 319 304 3718   0.000 000 000 0075    
c electron-muon magn. moment ratio                       206.766 9894          0.000 0054            
c electron-proton magn. moment ratio                     -658.210 6862         0.000 0066            
c electron to shielded proton magn. moment ratio         -658.227 5956         0.000 0071            
c electron-neutron magn. moment ratio                    960.920 50            0.000 23              
c electron-deuteron magn. moment ratio                   -2143.923 493         0.000 023             
c electron to shielded helion magn. moment ratio         864.058 255           0.000 010             
c electron gyromagn. ratio                               1.760 859 74e11       0.000 000 15e11       s^-1 T^-1
c electron gyromagn. ratio over 2 pi                     28 024.9532           0.0024                MHz T^-1
c muon mass                                              1.883 531 40e-28      0.000 000 33e-28      kg
c muon mass in u                                         0.113 428 9264        0.000 000 0030        u
c muon mass energy equivalent                            1.692 833 60e-11      0.000 000 29e-11      J
c muon mass energy equivalent in MeV                     105.658 3692          0.000 0094            MeV
c muon-electron mass ratio                               206.768 2838          0.000 0054            
c muon-tau mass ratio                                    5.945 92e-2           0.000 97e-2           
c muon-proton mass ratio                                 0.112 609 5269        0.000 000 0029        
c muon-neutron mass ratio                                0.112 454 5175        0.000 000 0029        
c muon molar mass                                        0.113 428 9264e-3     0.000 000 0030e-3     kg mol^-1
c muon Compton wavelength                                11.734 441 05e-15     0.000 000 30e-15      m
c muon Compton wavelength over 2 pi                      1.867 594 298e-15     0.000 000 047e-15     m
c muon magn. moment                                      -4.490 447 99e-26     0.000 000 40e-26      J T^-1
c muon magn. moment to Bohr magneton ratio               -4.841 970 45e-3      0.000 000 13e-3       
c muon magn. moment to nuclear magneton ratio            -8.890 596 98         0.000 000 23          
c muon magn. moment anomaly                              1.165 919 81e-3       0.000 000 62e-3       
c muon g factor                                          -2.002 331 8396       0.000 000 0012        
c muon-proton magn. moment ratio                         -3.183 345 118        0.000 000 089         
c tau mass                                               3.167 77e-27          0.000 52e-27          kg
c tau mass in u                                          1.907 68              0.000 31              u
c tau mass energy equivalent                             2.847 05e-10          0.000 46e-10          J
c tau mass energy equivalent in MeV                      1776.99               0.29                  MeV
c tau-electron mass ratio                                3477.48               0.57                  
c tau-muon mass ratio                                    16.8183               0.0027                
c tau-proton mass ratio                                  1.893 90              0.000 31              
c tau-neutron mass ratio                                 1.891 29              0.000 31              
c tau molar mass                                         1.907 68e-3           0.000 31e-3           kg mol^-1
c tau Compton wavelength                                 0.697 72e-15          0.000 11e-15          m
c tau Compton wavelength over 2 pi                       0.111 046e-15         0.000 018e-15         m
c proton mass                                            1.672 621 71e-27      0.000 000 29e-27      kg
c proton mass in u                                       1.007 276 466 88      0.000 000 000 13      u
c proton mass energy equivalent                          1.503 277 43e-10      0.000 000 26e-10      J
c proton mass energy equivalent in MeV                   938.272 029           0.000 080             MeV
c proton-electron mass ratio                             1836.152 672 61       0.000 000 85          
c proton-muon mass ratio                                 8.880 243 33          0.000 000 23          
c proton-tau mass ratio                                  0.528 012             0.000 086             
c proton-neutron mass ratio                              0.998 623 478 72      0.000 000 000 58      
c proton charge to mass quotient                         9.578 833 76e7        0.000 000 82e7        C kg^-1
c proton molar mass                                      1.007 276 466 88e-3   0.000 000 000 13e-3   kg mol^-1
c proton Compton wavelength                              1.321 409 8555e-15    0.000 000 0088e-15    m
c proton Compton wavelength over 2 pi                    0.210 308 9104e-15    0.000 000 0014e-15    m
c proton magn. moment                                    1.410 606 71e-26      0.000 000 12e-26      J T^-1
c proton magn. moment to Bohr magneton ratio             1.521 032 206e-3      0.000 000 015e-3      
c proton magn. moment to nuclear magneton ratio          2.792 847 351         0.000 000 028         
c proton g factor                                        5.585 694 701         0.000 000 056         
c proton-neutron magn. moment ratio                      -1.459 898 05         0.000 000 34          
c shielded proton magn. moment                           1.410 570 47e-26      0.000 000 12e-26      J T^-1
c shielded proton magn. moment to Bohr magneton ratio    1.520 993 132e-3      0.000 000 016e-3      
c shielded proton magn. moment to nuclear magneton ratio 2.792 775 604         0.000 000 030         
c proton magn. shielding correction                      25.689e-6             0.015e-6              
c proton gyromagn. ratio                                 2.675 222 05e8        0.000 000 23e8        s^-1 T^-1
c proton gyromagn. ratio over 2 pi                       42.577 4813           0.000 0037            MHz T^-1
c shielded proton gyromagn. ratio                        2.675 153 33e8        0.000 000 23e8        s^-1 T^-1
c shielded proton gyromagn. ratio over 2 pi              42.576 3875           0.000 0037            MHz T^-1
c proton rms charge radius                               0.8750e-15            0.0068e-15            m
c neutron mass                                           1.674 927 28e-27      0.000 000 29e-27      kg
c neutron mass in u                                      1.008 664 915 60      0.000 000 000 55      u
c neutron mass energy equivalent                         1.505 349 57e-10      0.000 000 26e-10      J
c neutron mass energy equivalent in MeV                  939.565 360           0.000 081             MeV
c neutron-electron mass ratio                            1838.683 6598         0.000 0013            
c neutron-muon mass ratio                                8.892 484 02          0.000 000 23          
c neutron-tau mass ratio                                 0.528 740             0.000 086             
c neutron-proton mass ratio                              1.001 378 418 70      0.000 000 000 58      
c neutron molar mass                                     1.008 664 915 60e-3   0.000 000 000 55e-3   kg mol^-1
c neutron Compton wavelength                             1.319 590 9067e-15    0.000 000 0088e-15    m
c neutron Compton wavelength over 2 pi                   0.210 019 4157e-15    0.000 000 0014e-15    m
c neutron magn. moment                                   -0.966 236 45e-26     0.000 000 24e-26      J T^-1
c neutron magn. moment to Bohr magneton ratio            -1.041 875 63e-3      0.000 000 25e-3       
c neutron magn. moment to nuclear magneton ratio         -1.913 042 73         0.000 000 45          
c neutron g factor                                       -3.826 085 46         0.000 000 90          
c neutron-electron magn. moment ratio                    1.040 668 82e-3       0.000 000 25e-3       
c neutron-proton magn. moment ratio                      -0.684 979 34         0.000 000 16          
c neutron to shielded proton magn. moment ratio          -0.684 996 94         0.000 000 16          
c neutron gyromagn. ratio                                1.832 471 83e8        0.000 000 46e8        s^-1 T^-1
c neutron gyromagn. ratio over 2 pi                      29.164 6950           0.000 0073            MHz T^-1
c deuteron mass                                          3.343 583 35e-27      0.000 000 57e-27      kg
c deuteron mass in u                                     2.013 553 212 70      0.000 000 000 35      u
c deuteron mass energy equivalent                        3.005 062 85e-10      0.000 000 51e-10      J
c deuteron mass energy equivalent in MeV                 1875.612 82           0.000 16              MeV
c deuteron-electron mass ratio                           3670.482 9652         0.000 0018            
c deuteron-proton mass ratio                             1.999 007 500 82      0.000 000 000 41      
c deuteron molar mass                                    2.013 553 212 70e-3   0.000 000 000 35e-3   kg mol^-1
c deuteron magn. moment                                  0.433 073 482e-26     0.000 000 038e-26     J T^-1
c deuteron magn. moment to Bohr magneton ratio           0.466 975 4567e-3     0.000 000 0050e-3     
c deuteron magn. moment to nuclear magneton ratio        0.857 438 2329        0.000 000 0092        
c deuteron-electron magn. moment ratio                   -4.664 345 548e-4     0.000 000 050e-4      
c deuteron-proton magn. moment ratio                     0.307 012 2084        0.000 000 0045        
c deuteron-neutron magn. moment ratio                    -0.448 206 52         0.000 000 11          
c deuteron rms charge radius                             2.1394e-15            0.0028e-15            m
c helion mass                                            5.006 412 14e-27      0.000 000 86e-27      kg
c helion mass in u                                       3.014 932 2434        0.000 000 0058        u
c helion mass energy equivalent                          4.499 538 84e-10      0.000 000 77e-10      J
c helion mass energy equivalent in MeV                   2808.391 42           0.000 24              MeV
c helion-electron mass ratio                             5495.885 269          0.000 011             
c helion-proton mass ratio                               2.993 152 6671        0.000 000 0058        
c helion molar mass                                      3.014 932 2434e-3     0.000 000 0058e-3     kg mol^-1
c shielded helion magn. moment                           -1.074 553 024e-26    0.000 000 093e-26     J T^-1
c shielded helion magn. moment to Bohr magneton ratio    -1.158 671 474e-3     0.000 000 014e-3      
c shielded helion magn. moment to nuclear magneton ratio -2.127 497 723        0.000 000 025         
c shielded helion to proton magn. moment ratio           -0.761 766 562        0.000 000 012         
c shielded helion to shielded proton magn. moment ratio  -0.761 786 1313       0.000 000 0033        
c shielded helion gyromagn. ratio                        2.037 894 70e8        0.000 000 18e8        s^-1 T^-1
c shielded helion gyromagn. ratio over 2 pi              32.434 1015           0.000 0028            MHz T^-1
c alpha particle mass                                    6.644 6565e-27        0.000 0011e-27        kg
c alpha particle mass in u                               4.001 506 179 149     0.000 000 000 056     u
c alpha particle mass energy equivalent                  5.971 9194e-10        0.000 0010e-10        J
c alpha particle mass energy equivalent in MeV           3727.379 17           0.000 32              MeV
c alpha particle-electron mass ratio                     7294.299 5363         0.000 0032            
c alpha particle-proton mass ratio                       3.972 599 689 07      0.000 000 000 52      
c alpha particle molar mass                              4.001 506 179 149e-3  0.000 000 000 056e-3  kg mol^-1
c Avogadro constant                                      6.022 1415e23         0.000 0010e23         mol^-1
c atomic mass constant                                   1.660 538 86e-27      0.000 000 28e-27      kg
c atomic mass constant energy equivalent                 1.492 417 90e-10      0.000 000 26e-10      J
c atomic mass constant energy equivalent in MeV          931.494 043           0.000 080             MeV
c Faraday constant                                       96 485.3383           0.0083                C mol^-1
c Faraday constant for conventional electric current     96 485.336            0.016                 C_90 mol^-1
c molar Planck constant                                  3.990 312 716e-10     0.000 000 027e-10     J s mol^-1
c molar Planck constant times c                          0.119 626 565 72      0.000 000 000 80      J m mol^-1
c molar gas constant                                     8.314 472             0.000 015             J mol^-1 K^-1
c Boltzmann constant                                     1.380 6505e-23        0.000 0024e-23        J K^-1
c Boltzmann constant in eV/K                             8.617 343e-5          0.000 015e-5          eV K^-1
c Boltzmann constant in Hz/K                             2.083 6644e10         0.000 0036e10         Hz K^-1
c Boltzmann constant in inverse meters per kelvin        69.503 56             0.000 12              m^-1 K^-1
c molar volume of ideal gas (273.15 K, 101.325 kPa)      22.413 996e-3         0.000 039e-3          m^3 mol^-1
c Loschmidt constant (273.15 K, 101.325 kPa)             2.686 7773e25         0.000 0047e25         m^-3
c molar volume of ideal gas (273.15 K, 100 kPa)          22.710 981e-3         0.000 040e-3          m^3 mol^-1
c Sackur-Tetrode constant (1 K, 100 kPa)                 -1.151 7047           0.000 0044            
c Sackur-Tetrode constant (1 K, 101.325 kPa)             -1.164 8677           0.000 0044            
c Stefan-Boltzmann constant                              5.670 400e-8          0.000 040e-8          W m^-2 K^-4
c first radiation constant                               3.741 771 38e-16      0.000 000 64e-16      W m^2
c first radiation constant for spectral radiance         1.191 042 82e-16      0.000 000 20e-16      W m^2 sr^-1
c second radiation constant                              1.438 7752e-2         0.000 0025e-2         m K
c Wien displacement law constant                         2.897 7685e-3         0.000 0051e-3         m K
c molar mass of carbon-12                                12e-3                 (exact)               kg mol^-1
c molar mass constant                                    1e-3                  (exact)               kg mol^-1
c conventional value of Josephson constant               483 597.9e9           (exact)               Hz V^-1
c conventional value of von Klitzing constant            25 812.807            (exact)               ohm
c standard atmosphere                                    101 325               (exact)               Pa
c standard acceleration of gravity                       9.806 65              (exact)               m s^-2
c Cu x unit                                              1.002 077 10e-13      0.000 000 29e-13      m
c Mo x unit                                              1.002 099 66e-13      0.000 000 53e-13      m
c Angstrom star                                          1.000 015 09e-10      0.000 000 90e-10      m
c lattice parameter of silicon                           543.102 122e-12       0.000 020e-12         m
c {220} lattice spacing of silicon                       192.015 5965e-12      0.000 0070e-12        m
c molar volume of silicon                                12.058 8382e-6        0.000 0024e-6         m^3 mol^-1
c electron volt                                          1.602 176 53e-19      0.000 000 14e-19      J
c unified atomic mass unit                               1.660 538 86e-27      0.000 000 28e-27      kg
c natural unit of velocity                               299 792 458           (exact)               m s^-1
c natural unit of action                                 1.054 571 68e-34      0.000 000 18e-34      J s
c natural unit of action in eV s                         6.582 119 15e-16      0.000 000 56e-16      eV s
c natural unit of mass                                   9.109 3826e-31        0.000 0016e-31        kg
c natural unit of energy                                 8.187 1047e-14        0.000 0014e-14        J
c natural unit of energy in MeV                          0.510 998 918         0.000 000 044         MeV
c natural unit of momentum                               2.730 924 19e-22      0.000 000 47e-22      kg m s^-1
c natural unit of momentum in MeV/c                      0.510 998 918         0.000 000 044         MeV/c
c natural unit of length                                 386.159 2678e-15      0.000 0026e-15        m
c natural unit of time                                   1.288 088 6677e-21    0.000 000 0086e-21    s
c atomic unit of charge                                  1.602 176 53e-19      0.000 000 14e-19      C
c atomic unit of mass                                    9.109 3826e-31        0.000 0016e-31        kg
c atomic unit of action                                  1.054 571 68e-34      0.000 000 18e-34      J s
c atomic unit of length                                  0.529 177 2108e-10    0.000 000 0018e-10    m
c atomic unit of energy                                  4.359 744 17e-18      0.000 000 75e-18      J
c atomic unit of time                                    2.418 884 326 505e-17 0.000 000 000 016e-17 s
c atomic unit of force                                   8.238 7225e-8         0.000 0014e-8         N
c atomic unit of velocity                                2.187 691 2633e6      0.000 000 0073e6      m s^-1
c atomic unit of momentum                                1.992 851 66e-24      0.000 000 34e-24      kg m s^-1
c atomic unit of current                                 6.623 617 82e-3       0.000 000 57e-3       A
c atomic unit of charge density                          1.081 202 317e12      0.000 000 093e12      C m^-3
c atomic unit of electric potential                      27.211 3845           0.000 0023            V
c atomic unit of electric field                          5.142 206 42e11       0.000 000 44e11       V m^-1
c atomic unit of electric field gradient                 9.717 361 82e21       0.000 000 83e21       V m^-2
c atomic unit of electric dipole moment                  8.478 353 09e-30      0.000 000 73e-30      C m
c atomic unit of electric quadrupole moment              4.486 551 24e-40      0.000 000 39e-40      C m^2
c atomic unit of electric polarizablity                  1.648 777 274e-41     0.000 000 016e-41     C^2 m^2 J^-1
c atomic unit of 1st hyperpolarizablity                  3.206 361 51e-53      0.000 000 28e-53      C^3 m^3 J^-2
c atomic unit of 2nd hyperpolarizablity                  6.235 3808e-65        0.000 0011e-65        C^4 m^4 J^-3
c atomic unit of magn. flux density                      2.350 517 42e5        0.000 000 20e5        T
c atomic unit of magn. dipole moment                     1.854 801 90e-23      0.000 000 16e-23      J T^-1
c atomic unit of magnetizability                         7.891 036 60e-29      0.000 000 13e-29      J T^-2
c atomic unit of permittivity                            1.112 650 056...e-10  (exact)               F m^-1
c joule-kilogram relationship                            1.112 650 056...e-17  (exact)               kg
c joule-inverse meter relationship                       5.034 117 20e24       0.000 000 86e24       m^-1
c joule-hertz relationship                               1.509 190 37e33       0.000 000 26e33       Hz
c joule-kelvin relationship                              7.242 963e22          0.000 013e22          K
c joule-electron volt relationship                       6.241 509 47e18       0.000 000 53e18       eV
c joule-atomic mass unit relationship                    6.700 5361e9          0.000 0011e9          u
c joule-hartree relationship                             2.293 712 57e17       0.000 000 39e17       E_h
c kilogram-joule relationship                            8.987 551 787...e16   (exact)               J
c kilogram-inverse meter relationship                    4.524 438 91e41       0.000 000 77e41       m^-1
c kilogram-hertz relationship                            1.356 392 66e50       0.000 000 23e50       Hz
c kilogram-kelvin relationship                           6.509 650e39          0.000 011e39          K
c kilogram-electron volt relationship                    5.609 588 96e35       0.000 000 48e35       eV
c kilogram-atomic mass unit relationship                 6.022 1415e26         0.000 0010e26         u
c kilogram-hartree relationship                          2.061 486 05e34       0.000 000 35e34       E_h
c inverse meter-joule relationship                       1.986 445 61e-25      0.000 000 34e-25      J
c inverse meter-kilogram relationship                    2.210 218 81e-42      0.000 000 38e-42      kg
c inverse meter-hertz relationship                       299 792 458           (exact)               Hz
c inverse meter-kelvin relationship                      1.438 7752e-2         0.000 0025e-2         K
c inverse meter-electron volt relationship               1.239 841 91e-6       0.000 000 11e-6       eV
c inverse meter-atomic mass unit relationship            1.331 025 0506e-15    0.000 000 0089e-15    u
c inverse meter-hartree relationship                     4.556 335 252 760e-8  0.000 000 000 030e-8  E_h
c hertz-joule relationship                               6.626 0693e-34        0.000 0011e-34        J
c hertz-kilogram relationship                            7.372 4964e-51        0.000 0013e-51        kg
c hertz-inverse meter relationship                       3.335 640 951...e-9   (exact)               m^-1
c hertz-kelvin relationship                              4.799 2374e-11        0.000 0084e-11        K
c hertz-electron volt relationship                       4.135 667 43e-15      0.000 000 35e-15      eV
c hertz-atomic mass unit relationship                    4.439 821 667e-24     0.000 000 030e-24     u
c hertz-hartree relationship                             1.519 829 846 006e-16 0.000 000 000 010e-16 E_h
c kelvin-joule relationship                              1.380 6505e-23        0.000 0024e-23        J
c kelvin-kilogram relationship                           1.536 1808e-40        0.000 0027e-40        kg
c kelvin-inverse meter relationship                      69.503 56             0.000 12              m^-1
c kelvin-hertz relationship                              2.083 6644e10         0.000 0036e10         Hz
c kelvin-electron volt relationship                      8.617 343e-5          0.000 015e-5          eV
c kelvin-atomic mass unit relationship                   9.251 098e-14         0.000 016e-14         u
c kelvin-hartree relationship                            3.166 8153e-6         0.000 0055e-6         E_h
c electron volt-joule relationship                       1.602 176 53e-19      0.000 000 14e-19      J
c electron volt-kilogram relationship                    1.782 661 81e-36      0.000 000 15e-36      kg
c electron volt-inverse meter relationship               8.065 544 45e5        0.000 000 69e5        m^-1
c electron volt-hertz relationship                       2.417 989 40e14       0.000 000 21e14       Hz
c electron volt-kelvin relationship                      1.160 4505e4          0.000 0020e4          K
c electron volt-atomic mass unit relationship            1.073 544 171e-9      0.000 000 092e-9      u
c electron volt-hartree relationship                     3.674 932 45e-2       0.000 000 31e-2       E_h
c atomic mass unit-joule relationship                    1.492 417 90e-10      0.000 000 26e-10      J
c atomic mass unit-kilogram relationship                 1.660 538 86e-27      0.000 000 28e-27      kg
c atomic mass unit-inverse meter relationship            7.513 006 608e14      0.000 000 050e14      m^-1
c atomic mass unit-hertz relationship                    2.252 342 718e23      0.000 000 015e23      Hz
c atomic mass unit-kelvin relationship                   1.080 9527e13         0.000 0019e13         K
c atomic mass unit-electron volt relationship            931.494 043e6         0.000 080e6           eV
c atomic mass unit-hartree relationship                  3.423 177 686e7       0.000 000 023e7       E_h
c hartree-joule relationship                             4.359 744 17e-18      0.000 000 75e-18      J
c hartree-kilogram relationship                          4.850 869 60e-35      0.000 000 83e-35      kg
c hartree-inverse meter relationship                     2.194 746 313 705e7   0.000 000 000 015e7   m^-1
c hartree-hertz relationship                             6.579 683 920 721e15  0.000 000 000 044e15  Hz
c hartree-kelvin relationship                            3.157 7465e5          0.000 0055e5          K
c hartree-electron volt relationship                     27.211 3845           0.000 0023            eV
c hartree-atomic mass unit relationship                  2.921 262 323e-8      0.000 000 019e-8      u

c===============================================================================

c.... Dimensioned arguments.

      dimension ind     (1)           ! Index in array of local constants.
      dimension pgcon   (1)           ! Constant pgcon(n) = constant(ind(n)).

c.... Local variables.

      common /laptconm/
     &  pga(1),   pgabi85,  pgad,     pgalpha,  pgalphai, pgamtoev,
     &  pgamtoht, pgamtohz, pgamtoj,  pgamtok,  pgamtokg, pgamtorm,
     &  pgamu,    pgamud,   pgamue,   pgamumu,  pgamun,   pgamup,
     &  pgastar,  pgasube,  pgasubmu, pgatm,    pgatomx,  pgbohr,
     &  pgc,      pgcsub1,  pgcsub2,  pgcsub2d, pgcunit,  pgcuxun,
     &  pgdrohm,  pge,      pgebase,  pgec,     pgeoh,    pgeome,
     &  pgeomp,   pgepsvac, pgerad,   pgevtoam, pgevtoht, pgevtohz,
     &  pgevtoj,  pgevtok,  pgevtokg, pgevtorm, pgfar,    pgg,
     &  pggamma,  pggamp,   pggampb,  pggampp,  pggamppb, pggolden,
     &  pggsube,  pggsubmu, pggsubn,  pgh,      pghart,   pghartev
      common /laptconm/
     &  pghbar,   pghbarc,  pghbarok, pghlen,   pghmass,  pghok,
     &  pghtime,  pghttoam, pghttoev, pghttohz, pghttoj,  pghttok,
     &  pghttokg, pghttorm, pghyfsph, pghztoam, pghztoev, pghztoht,
     &  pghztoj,  pghztok,  pghztokg, pghztorm, pgjfvr,   pgjtoamu,
     &  pgjtoev,  pgjtoht,  pgjtohz,  pgjtok,   pgjtokg,  pgjtorm,
     &  pgk,      pgkd,     pgkgtoam, pgkgtoev, pgkgtoht, pgkgtohz,
     &  pgkgtoj,  pgkgtok,  pgkgtorm, pgktoam,  pgktoev,  pgktoht,
     &  pgktohz,  pgktoj,   pgktokg,  pgktorm,  pglamce,  pglamceb,
     &  pglamcn,  pglamcnb, pglamcp,  pglamcpb, pglosch,  pgmagfq,
     &  pgmdev,   pgmdj,    pgmeev,   pgmej,    pgmmrde,  pgmmrdp
      common /laptconm/
     &  pgmmremu, pgmmrep,  pgmmrmup, pgmmrne,  pgmmrnp,  pgmmuev,
     &  pgmmuj,   pgmnev,   pgmnj,    pgmolh,   pgmolhc,  pgmoxun,
     &  pgmpev,   pgmpj,    pgmsubd,  pgmsube,  pgmsubmu, pgmsubn,
     &  pgmsubp,  pgmub,    pgmube,   pgmubf,   pgmubk,   pgmubw,
     &  pgmud,    pgmudb,   pgmudn,   pgmue,    pgmueb,   pgmuen,
     &  pgmumu,   pgmumub,  pgmumun,  pgmun,    pgmune,   pgmunf,
     &  pgmunk,   pgmunt,   pgmuntb,  pgmuntn,  pgmunw,   pgmup,
     &  pgmupb,   pgmupn,   pgmupp,   pgmuppb,  pgmuppn,  pgmuvac,
     &  pgnsuba,  pgombi85, pgqcirc,  pgqcirch, pgqhc,    pgqhr,
     &  pgratde,  pgratdp,  pgratea,  pgrated,  pgratemu, pgratep
      common /laptconm/
     &  pgratmue, pgratne,  pgratnp,  pgratpe,  pgratpmu, pgrmold,
     &  pgrmole,  pgrmtoam, pgrmtoev, pgrmtoht, pgrmtohz, pgrmtoj,
     &  pgrmtok,  pgrmtokg, pgrydb,   pgrydbc,  pgrydbe,  pgrydbev,
     &  pgsate,   pgsatez,  pgsb,     pgsbd,    pgsia,    pgsid220,
     &  pgsigw,   pgsivolm, pgtemp,   pgtempt,  pgthomx,  pgubb,
     &  pgubbd,   pgv76bi,  pgvmolt,  pgvmoltz, pgwien,   pgwiend,
     &  pi

      common /laptconm/ indx          ! Index in external array.
      common /laptconm/ n             ! Index in external array.
cbugc***DEBUG begins.
cbug 9901 format (/ 'aptconm finding physical constants.  np=',i4 /
cbug     &  ('  n=',i4,' ind=',i3))
cbug      write ( 3, 9901) np, (n, ind(n), n = 1, np)
cbugc***DEBUG ends.

c.... The data.

c....                          GENERAL CONSTANTS

c....                         Numerical Constants

c.... Ratio of circumference to diameter of circle, pi.
      pi       = 3.14159265358979323  ! Dimensionless.    18 digits.

c.... Euler-Masceroni constant, gamma.
      pggamma  = 0.57721566490153386  ! Dimensionless.    17 digits.

c.... Golden Ratio, Phi = 0.5 * (1 + 5**0.5).
      pggolden = 1.61803398874989484  ! Dimensionless.    18 digits.

c.... Base of natural logarithms, e.
      pgebase  = 2.71828182845904523  ! Dimensionless.    18 digits.

c....                         Universal Constants

c.... Speed of light in vacuum, c.
      pgc      = 299792458.0          ! m / s.                exact.

c.... Unit system constant, c_unit.
      pgcunit  = 1.0000000000000e-07  ! N / A**2 = H / m.     exact.

c.... Permeability of vacuum, mu_0 = 4 * pi * c_unit.
      pgmuvac  = 12.566370614359e-07  ! N / A**2 = H / m.     0 ppt.

c.... Permittivity of vacuum, epsilon_0 = 1 / (mu_0 * c**2).
      pgepsvac = 8.8541878176203e-12  ! F / m.                0 ppt.

c.... Newtonian constant of gravitation, G.
      pgg      = 6.6742300000000e-11  ! m**3 / (kg s**2).    14 ppm.

c.... Boltzmann constant in energy and Kelvin units, k_d = kd.
      pgkd     = 1.3806503000000e-23  ! J / K.              1.7 ppm.

c.... Boltzmann constant in energy and electron volt units, k.
      pgk      = 1.6021764620000e-19  ! J / eV.              39 ppb.

c.... Planck constant, h.
      pgh      = 6.6260687600000e-34  ! J s.                 78 ppb.

c.... Planck constant in electron volts, h / k.
      pghok    = 4.1356672700000e-15  ! eV s.                39 ppb.

c.... Planck constant / (2.0 * pi), hbar.
      pghbar   = 1.0545715960000e-34  ! J s.                 78 ppb.

c.... Planck constant / (2.0 * pi) in electron volts, hbar / k.
      pghbarok = 6.5821188900000e-16  ! eV s.                39 ppb.

c.... Planck constant * velocity of light / (2.0 * pi), hbar * c.
      pghbarc  = 3.1615261090182e-26  ! J m.                 78 ppb.

c.... Planck mass, m_P = (hbar * c / G)**0.5.
      pghmass  = 2.1764450000000e-08  ! kg.                   7 ppm.

c.... Planck length, l_P = hbar / (m_P * c).
      pghlen   = 1.6162460000000e-35  ! m.                    7 ppm.

c.... Planck time, t_P = l_P / c.
      pghtime  = 5.3912180000000e-44  ! s.                    7 ppm.

c....                      Electromagnetic Constants

c.... Elementary charge, e.
      pge      = 1.6021764620000e-19  ! C.                   39 ppb.

c.... Elementary charge * velocity of light, e * c.
      pgec     = 4.8032041970000e-11  ! C m / s.             39 ppb.

c.... Elementary charge / Planck constant, e / h.
      pgeoh    = 2.4179894910000e+14  ! A / J.               78 ppb.

c.... Magnetic flux quantum, Phi_0 = h / (2 * e).
      pgmagfq  = 2.0678336360000e-15  ! Wb = V s.            39 ppb.

c.... Josephson frequency-voltage ratio, 2 * e / h.
      pgjfvr   = 4.8359789800000e+14  ! Hz / V.              39 ppb.

c.... Quantized Hall conductance, e**2 / h.
      pgqhc    = 3.8740458480000e-05  ! S = A / V.          3.7 ppb.

c.... Quantized Hall resistance, R_H = h / e**2.
c....   AKA von Klitzing constant.
      pgqhr    = 25812.807572         ! ohm.                3.7 ppb.

c.... Bohr magneton, mu_B = e * hbar / (2 * m_e).
      pgmub    = 9.2740089900000e-24  ! J / T.               40 ppb.

c.... Bohr magneton in electron volts, mu_B / k.
      pgmube   = 5.7883817490000e-05  ! eV / T.             7.3 ppb.

c.... Bohr magneton in frequency units, mu_B / h.
      pgmubf   = 1.3996246240000e+10  ! Hz / T.              40 ppb.

c.... Bohr magneton in wavenumbers, mu_B / (h * c).
      pgmubw   = 46.6864521           ! (1 / m) / T.         40 ppb.

c.... Bohr magneton in Kelvins, mu_B / k_d.
      pgmubk   = 0.6717131            ! K / T.              1.7 ppm.

c.... Nuclear magneton, mu_N = e * hbar / (2 * m_p).
      pgmun    = 5.0507831700000e-27  ! J / T.               40 ppb.

c.... Nuclear magneton in electron volts, mu_N / k.
      pgmune   = 3.1524512380000e-08  ! eV / T.             7.6 ppb.

c.... Nuclear magneton in frequency units, mu_N / h.
      pgmunf   = 7622593.96           ! Hz / T.              40 ppm.

c.... Nuclear magneton in wavenumbers, mu_N / (h * c).
      pgmunw   = 0.0254262366         ! (1 / m) / T.         40 ppm.

c.... Nuclear magneton in Kelvins, mu_N / k_d.
      pgmunk   = 3.6582638000000e-04  ! K / T.              1.7 ppm.

c....                          ATOMIC CONSTANTS

c....                               General

c.... Fine-structure constant, alpha = mu_0 * c * e**2 / (2 * h).
      pgalpha  = 0.007297352535       ! Dimensionless.      0.7 ppb.

c.... Inverse fine-structure constant, 1 / alpha.
      pgalphai = 137.035999710        ! Dimensionless.      0.7 ppb.

c.... Rydberg constant in wavenumbers, R_infinity = Ry.
      pgrydb   = 10973731.568549      ! 1 / m.              7.6 ppt.

c.... Rydberg constant in frequency units, R_infinity * c.
      pgrydbc  = 3.2898419603680e+15  ! Hz.                 7.6 ppt.

c.... Rydberg constant in energy units, R_infinity * h * c.
      pgrydbe  = 2.1798719000000e-18  ! J.                   78 ppb.

c.... Rydberg constant in electron volts, R_infinity * h * c / pgk.
      pgrydbev = 13.60569172          ! eV.                  39 ppb.

c.... Bohr radius, a_0 = alpha / (4 * pi * r_infinity).
      pgbohr   = 5.2917720830000e-11  ! m.                  3.7 ppb.

c.... Atomic cross section, pi * a_0**2.
      pgatomx  = 8.7973553430000e-21  ! m**2.               7.4 ppb.

c.... Hartree energy, E_h = 2 * R_infinity * h * c.
c....                         = e**2 / (4 * pi * epsilon_0 * a_0).
      pghart   = 4.3597438100000e-18  ! J.                   78 ppb.

c.... Hartree energy in electron volts, E_h / k.
      pghartev = 27.2113834           ! eV.                  39 ppb.

c.... Quantum of circulation, h / m_e.
      pgqcirc  = 7.2738950320000e-04  ! m**2 / s.           7.3 ppb.

c.... Quantum of circulation, h / (2 * m_e).
      pgqcirch = 3.6369475160000e-04  ! m**2 / s.           7.3 ppb.

c....                              Electron

c.... Electron mass, m_e.
      pgmsube  = 9.1093818800000e-31  ! kg.                  79 ppb.

c.... Electron molar mass, M_e.
      pgamue   = 5.4857991100000e-07  ! kg / mol.           2.1 ppb.

c.... Electron mass in energy units.
      pgmej    = 8.1871041400000e-14  ! J.                   79 ppb.

c.... Electron mass in electron volts.
      pgmeev   = 510998.902           ! eV.                  40 ppb.

c.... Electron-muon mass ratio, m_e / m_mu.
      pgratemu = 0.00483633210        ! Dimensionless.       30 ppb.

c.... Electron-proton mass ratio, m_e / m_p.
      pgratep  = 0.0005446170232      ! Dimensionless.      2.1 ppb.

c.... Electron-deuteron mass ratio, m_e / m_d.
      pgrated  = 0.00027244371170     ! Dimensionless.      2.1 ppb.

c.... Electron-alpha-particle mass ratio, m_e / m_alpha.
      pgratea  = 0.00013709335611     ! Dimensionless.      2.1 ppb.

c.... Electron specific (negative) charge, e / m_e.
      pgeome   = 1.7588201740000e+11  ! C / kg.              40 ppb.

c.... Electron Compton wavelength, lambda_C = h / (m_e * c).
      pglamce  = 2.4263102150000e-12  ! m.                  7.3 ppb.

c.... Electron Compton wavelength / (2.0 * pi), lambdabar_C.
      pglamceb = 3.8615926420000e-13  ! m.                  7.3 ppb.

c.... Electron classical radius, r_e.
      pgerad   = 2.8179402850000e-15  ! m.                   11 ppb.

c.... Thomson cross section, sigma_e.
      pgthomx  = 0.6652458540000e-28  ! m**2.                22 ppb.

c.... Electron magnetic moment, mu_e.
      pgmue    = 9.2847636200000e-24  ! J / T.               40 ppb.

c.... Electron magnetic moment in Bohr magnetons, mu_e / mu_B.
      pgmueb   = 1.00115965218085     ! Dimensionless.      0.8 ppt.

c.... Electron magnetic moment in nuclear magnetons, mu_e / mu_N.
      pgmuen   = 1838.2819660         ! Dimensionless.      2.1 ppb.

c.... Electron magnetic moment anomaly, a_e.
      pgasube  = 0.00115965218085     ! Dimensionless.      0.8 ppb.

c.... Electron g-factor, g_e = 2 * (1 + a_e).
      pggsube  = 2.00231930436170     ! Dimensionless.      0.8 ppt.

c.... Electron-muon magnetic moment ratio, mu_e / mu_mu.
      pgmmremu = 206.7669720          ! Dimensionless.      2.1 ppb.

c.... Electron-proton magnetic moment ratio, mu_e / mu_p.
      pgmmrep  = 658.2106875          ! Dimensionless.       10 ppb.

c....                                Muon

c.... Muon mass, m_mu.
      pgmsubmu = 1.8835310900000e-28  ! kg.                  84 ppb.

c.... Muon molar mass, M_mu.
      pgamumu  = 0.0001134289168      ! kg / mol.            30 ppb.

c.... Muon mass in energy units.
      pgmmuj   = 1.6928333200000e-11  ! J.                   84 ppb.

c.... Muon mass in electron volts.
      pgmmuev  = 1.0565835680000e+08  ! eV.                  49 ppb.

c.... Muon-electron mass ratio, m_mu / m_e.
      pgratmue = 206.7682657          ! Dimensionless.       30 ppb.

c.... Muon magnetic moment, mu_mu.
      pgmumu   = 4.4904481300000e-26  ! J / T.               49 ppb.

c.... Muon magnetic moment in Bohr magnetons, mu_mu / mu_B.
      pgmumub  = 0.00484197085        ! Dimensionless.       30 ppb.

c.... Muon magnetic moment in nuclear magnetons, mu_mu / mu_N.
      pgmumun  = 8.89059770           ! Dimensionless.       30 ppb.

c.... Muon magnetic moment anomaly, a_mu.
      pgasubmu = 0.00116591602        ! Dimensionless.     0.55 ppm.

c.... Muon g-factor, g_mu = 2 * (1 + a_mu).
      pggsubmu = 2.0023318320         ! Dimensionless.     0.64 ppb.

c.... Muon-proton magnetic moment ratio, mu_mu / mu_p.
      pgmmrmup = 3.18334539           ! Dimensionless.       32 ppb.

c....                               Proton

c.... Proton mass, m_p.
      pgmsubp  = 1.6726215800000e-27  ! kg.                  79 ppb.

c.... Proton molar mass, M_p.
      pgamup   = 0.00100727646688     ! kg / mol.          0.13 ppb.

c.... Proton mass in energy units.
      pgmpj    = 1.5032773100000e-10  ! J.                   79 ppb.

c.... Proton mass in electron volts.
      pgmpev   = 9.3827199800000e+08  ! eV.                  40 ppb.

c.... Proton-electron mass ratio, m_p / m_e.
      pgratpe  = 1836.1526675         ! Dimensionless.      2.1 ppb.

c.... Proton-muon mass ratio, m_p / m_mu.
      pgratpmu = 8.88024408           ! Dimensionless.       30 ppb.

c.... Proton specific charge, e / m_p.
      pgeomp   = 9.5788340800000e+07  ! C / kg.              40 ppb.

c.... Proton Compton wavelength, lambda_C,p = h / (m_p * c).
      pglamcp  = 1.3214098470000e-15  ! m.                  7.6 ppb.

c.... Proton Compton wavelength / (2.0 * pi), lambdabar_C,p.
      pglamcpb = 2.1030890890000e-16  ! m.                  7.6 ppb.

c.... Proton magnetic moment, mu_p.
      pgmup    = 1.4106066330000e-26  ! J / T.               41 ppb.

c.... Proton magnetic moment in Bohr magnetons, mu_p / mu_B.
      pgmupb   = 0.001521032203       ! Dimensionless.       10 ppb.

c.... Proton magnetic moment in nuclear magnetons, mu_p / mu_N.
      pgmupn   = 2.792847337          ! Dimensionless.       10 ppb.

c.... Proton magnetic moment diamagnetic shielding correction, sigma_water,
c....   in pure water, spherical sample, 298.15 K, 1 - mu'_p / mu_p.
      pgsigw   = 2.5687000000000e-05  ! Dimensionless.      570 ppm.

c.... Shielded proton moment, mu'_p (in water, spherical, 298.15 K).
      pgmupp   = 1.4105703990000e-26  ! J / T.               42 ppb.

c.... Shielded proton moment in Bohr magnetons, mu'_p / mu_B.
      pgmuppb  = 0.001520993132       ! Dimensionless.       11 ppb.

c.... Shielded proton moment in nuclear magnetons, mu'_p / mu_N.
      pgmuppn  = 2.792775597          ! Dimensionless.       11 ppb.

c.... Proton gyromagnetic ratio, gamma_p = 2 * mu_p / hbar.
      pggamp   = 2.6752221200000e+08  ! 1 / (s T).           41 ppb.

c.... Proton gyromagnetic ratio / (2.0 * pi), gamma_p / (2 * pi).
      pggampb  = 4.2577482500000e+07  ! Hz / T.              41 ppb.

c.... Shielded proton gyromagnetic ratio, gamma'_p = 2 * mu'_p / hbar.
      pggampp  = 2.6751534100000e+08  ! 1 / (s T).           42 ppb.

c.... Shielded proton gyromagnetic ratio / (2.0 * pi), gamma'_p / (2 * pi).
      pggamppb = 4.2576388800000e+07  ! Hz / T.              42 ppb.

c....                               Neutron

c.... Neutron mass, m_n.
      pgmsubn  = 1.6749271600000e-27  ! kg.                  79 ppb.

c.... Neutron molar mass, M_n.
      pgamun   = 0.00100866491578     ! kg / mol.          0.54 ppb.

c.... Neutron mass in energy units.
      pgmnj    = 1.5053494600000e-10  ! J.                   79 ppb.

c.... Neutron mass in electron volts.
      pgmnev   = 9.3956533000000e+08  ! eV.                  40 ppb.

c.... Neutron-electron mass ratio, m_n / m_e.
      pgratne  = 1838.6836550         ! Dimensionless.      2.2 ppb.

c.... Neutron-proton mass ratio, m_n / m_p.
      pgratnp  = 1.00137841887        ! Dimensionless.     0.58 ppb.

c.... Neutron Compton wavelength, lambda_C,n = h / (m_n * c).
      pglamcn  = 1.3195908980000e-15  ! m.                  7.6 ppb.

c.... Neutron Compton wavelength / (2.0 * pi), lambdabar_C,n.
      pglamcnb = 2.1001941420000e-16  ! m.                  7.6 ppb.

c.... Neutron magnetic moment, mu_n (scaler, opposite from proton).
      pgmunt   = 9.6623640000000e-27  ! J / T.             0.24 ppm.

c.... Neutron magnetic moment in Bohr magnetons, mu_n / mu_B.
      pgmuntb  = 0.00104187563        ! Dimensionless.     0.24 ppm.

c.... Neutron magnetic moment in nuclear magnetons, mu_n / mu_N.
      pgmuntn  = 1.91304272           ! Dimensionless.     0.24 ppm.

c.... Neutron-electron magnetic moment ratio, mu_n / mu_e.
      pgmmrne  = 0.00104066882        ! Dimensionless.     0.24 ppm.

c.... Neutron-proton magnetic moment ratio, mu_n / mu_p.
      pgmmrnp  = 0.68497934           ! Dimensionless.     0.24 ppm.

c....                              Deuteron

c.... Deuteron mass, m_d.
      pgmsubd  = 3.3435830900000e-27  ! kg.                  79 ppb.

c.... Deuteron molar mass, M_d.
      pgamud   = 0.00201355321271     ! kg / mol.          0.17 ppb.

c.... Deuteron mass in energy units.
      pgmdj    = 3.0050626200000e-10  ! J.                   79 ppb.

c.... Deuteron mass in electron volts.
      pgmdev   = 1.8756127620000e+09  ! eV.                  40 ppb.

c.... Deuteron-electron mass ratio, m_d / m_e.
      pgratde  = 3670.4829550         ! Dimensionless.      2.1 ppb.

c.... Deuteron-proton mass ratio, m_d / m_p.
      pgratdp  = 1.99900750083        ! Dimensionless.     0.17 ppb.

c.... Deuteron magnetic moment, mu_d (scaler).
      pgmud    = 4.3307345700000e-27  ! J / T.               42 ppb.

c.... Deuteron magnetic moment in Bohr magnetons, mu_d / mu_B.
      pgmudb   = 0.0004669754556      ! Dimensionless.       11 ppb.

c.... Deuteron magnetic moment in nuclear magnetons, mu_d / mu_N.
      pgmudn   = 0.8574382284         ! Dimensionless.       11 ppb.

c.... Deuteron-electron magnetic moment ratio, mu_d / mu_e.
      pgmmrde  = 0.0004664345537      ! Dimensionless.       11 ppb.

c.... Deuteron-proton magnetic moment ratio, mu_d / mu_p.
      pgmmrdp  = 0.3070122083         ! Dimensionless.       15 ppb.


c....                     PHYSICO-CHEMICAL CONSTANTS

c.... Avogadro constant, N_A = L = 12 / mass of C-12.
      pgnsuba  = 6.0221419900000e+23  ! # / mol = amu / g    79 ppb.

c.... Atomic mass constant, m_u = 1 / N_A = 1/12 mass of C-12.
      pgamu    = 1.6605387300000e-27  ! kg / amu.            79 ppb.

c.... Faraday constant, F.
      pgfar    = 96485.3415           ! C / mol.             40 ppb.

c.... Molar Planck constant, N_A * h.
      pgmolh   = 3.9903126890000e-10  ! J s / mol.          7.6 ppb.

c.... Molar Planck constant * c, N_A * h * c.
      pgmolhc  = 0.11962656492        ! J m / mol.          7.6 ppb.

c.... Molar gas constant, R.
      pgrmold  = 8.314472             ! J / (mol K).        1.7 ppm.

c.... Molar gas constant, R.
      pgrmole  = 96485.3415           ! J / (mol eV).        40 ppb.

c.... Molar volume (ideal gas), V_m = R*T/p.
c....  T = pgtempt = 273.15K,  pgtemp = 0.023538270 eV = 2.353838270e-5 keV,
c....   p = pgatm = 101325 Pa = 1.10325e-10 jerk / cm**3.
      pgvmolt  = 0.022413996          ! m**3 / mol.         1,7 ppm.


c.... Molar volume (ideal gas), V_m = R*T/p.
c....   T = pgtempt = 273.15K,  pgtemp = 0.023538270 eV = 2.353838270e-5 keV,
c....   p = 0.986923 atm = 100000 Pa = 1.0e-10 jerk / cm**3.
      pgvmoltz = 0.022710981          ! m**3 / mol.         1.7 ppm.


c.... Loschmidt constant, n_0 = N_A / V_m.
c....   T = 273.15 K = 0.0 Celsius = 0.023538270 eV = 2.353838270e-5 keV,
c....   p = 101325.0 Pa = 1.0 atm = 1.01325e-10 jerk / cm**3.
      pglosch  = 2.6867775000000e+25  ! # / m**3.           1.7 ppm.

c.... Sackur-Tetrode constant, S_0 / R.
c....   The entropy of an ideal monatomic gas of relative atomic weight A is
c....   S = S_0 + 1.5 * R * alog (A) - R * alog (p / p_0) +
c....       2.5 * R * alog (T / K).
c....   T_1 = 1 K, p_0 = 101325 Pa.
      pgsate   = -1.1648678           ! Dimensionless.      3.7 ppm.

c....   T_1 = 1 K, p_0 = 100000 Pa.
      pgsatez  = -1.1517048           ! Dimensionless.      3.8 ppm.

c.... Black body constant, u_bb.
      pgubbd   = 1.1650730000000e-16  ! J / (m**3 K**4).     34 ppm.

c.... Black body constant, u_bb.
      pgubb    = 2.1127638            ! J / (m**3 eV**4).     3 ppm.

c.... Radiation energy constant, u_a.
      pgad     = 7.5659100000000e-16  ! J / (m**3 K**4).     34 ppm.

c.... Radiation energy constant, u_a.
      pga(1)   = 13.72016             ! J / (m**3 eV**4).     3 ppm.

c.... Stefan-Boltzmann constant, sigma.
      pgsbd    = 5.6704000000000e-08  ! W / (m**2 K**4).      7 ppm.

c.... Stefan-Boltzmann constant, sigma.
      pgsb     = 1.0283012000000e+09  ! W / (m**2 eV**4).     3 ppm.

c.... First radiation constant, c_1 = 2.0 * pi * h * c**2.
      pgcsub1  = 3.7417710700000e-16  ! W m**2.              78 ppb.

c.... Second radiation constant in Kelvin units, c_2 = h * c / kd.
      pgcsub2d = 0.014387752          ! m K.                1.7 ppm.

c.... Second radiation constant in electron volt units, c_2 = h * c / k.
      pgcsub2  = 1.2398418570000e-06  ! eV m.                39 ppb.

c.... Wien displacement law constant in Kelvin units, b = lambda_max * T.
      pgwiend  = 0.0028977686         ! m K.                1.7 ppm.

c.... Wien displacement law constant in electron volt units, b.
      pgwien   = 2.4971076000000e-07  ! eV m.              0.30 ppm.


c....                           STANDARD VALUES

c.... Standard acceleration of gravity, g_n.
      pggsubn  = 9.80665              ! m / s**2.             exact.

c.... Standard atmosphere, p_0 = 1 atmosphere.
      pgatm    = 101325.0             ! Pa.                   exact.

c.... Standard temperature, T_0 = 0.0 Celsius.
      pgtempt  = 273.15               ! K.                    exact.

c.... Standard temperature, T_0 = 0.0 Celsius.
      pgtemp   = 0.023538270          ! eV.                 1.7 ppm.

c....                 "As-maintained" Electrical Standards

c.... BIPM maintained ohm, omega_69-BI (1 Jan 1985) = omega_BI85.
      pgombi85 = 0.999998437          ! ohm.                 50 ppb.

c.... Drift rate of omega_69-BI, d[omega_69-BI] / dt.
      pgdrohm  = -5.660000000000e-08  ! ohm / yr?                  ?

c.... BIPM maintained volt, V_76-BI = 4.83594e+14 * h / (2 * e).
      pgv76bi  = 0.99999241           ! V.                 0.30 ppm.

c.... BIPM maintained ampere, A_BI85 = V_76-BI / omega_69-BI.
      pgabi85  = 0.99999397           ! A.                 0.30 ppm.

c....                           X-Ray Standards

c.... Cu x-unit, xu_CuKalpha_1:
c....   lambda_CuKalpha_1 = 1537.400 xu.
      pgcuxun  = 1.0020770300000e-13  ! m.                 0.28 ppm.

c.... Mo x-unit, xu_MoKalpha_1:
c....   lambda_MoKalpha_1 = 707.831 xu.
      pgmoxun  = 1.0020995900000e-13  ! m.                 0.53 ppm.

c.... A^*:  lambda_WKalpha_1 = 0.209100 A^*.
      pgastar  = 1.0000150100000e-10  ! m.                 0.90 ppm.

c.... Lattice spacing of Si, a (in vacuum, 295.65 K).
      pgsia    = 5.4310208800000e-10  ! m.                   29 ppb.

c.... Lattice spacing of Si, d_220 (in vacuum, 295.65 K) = a / 8**0.5.
      pgsid220 = 1.9201558450000e-10  ! m.                   29 ppb.

c.... Molar volume of Si, V_m (Si) = M(Si) / rho(Si) = N_A a**3 / 8.
      pgsivolm = 12.058836900000e-06  ! m**3 / mol.        0.12 ppm.


c....                      ENERGY CONVERSION FACTORS

c.... Basis of conversion factors for energy equivalents:
c....  E = mass * c**2 = h * c / lambda = h * frequency = k * temperature
c....  1 eV = (e/C) * J
c....  1 u = m_u = (1/12) * (mass of carbon-12) = kg/(mole * Avogadro number)
c....  E_h = 2 * R_infinity * h * c = alpha*82 * m_e * c**2

c....                  Conversion to kg mass units

c.... Equivalent:  1 / L.
      pgamtokg = 1.6605387300000e-27  ! kg / amu.            79 ppb.

c.... Equivalent:  1 / c**2.
      pgjtokg  = 1.1126500560536e-17  ! kg / J.               0 ppt.

c.... Equivalent:  kd / c**2.
      pgktokg  = 1.5361807000000e-40  ! kg / K.             1.7 ppm.

c.... Equivalent:  k / c**2.
      pgevtokg = 1.7826617310000e-36  ! kg / eV.             39 ppb.

c.... Equivalent:  h / c.
      pgrmtokg = 2.2102186300000e-42  ! kg / (1/m).          77 ppb.

c.... Equivalent:  h / c**2.
      pghztokg = 7.3724957800000e-51  ! kg / Hz.             79 ppb.

c.... Equivalent:  2 * Ry * h / c.
      pghttokg = 4.8508691900000e-35  ! kg / hartree.        78 ppb.

c....                   Conversion to atomic mass units

c.... Equivalent:  L.
      pgkgtoam = 6.0221419900000e+26  ! amu / kg.            79 ppb.

c.... Equivalent:  L / c**2.
      pgjtoamu = 6.7005366200000e+09  ! amu / J.             79 ppb.

c.... Equivalent:  L * kd / c**2.
      pgktoam  = 9.2510980000000e-14  ! amu / K.            1.7 ppm.

c.... Equivalent:  L * k / c**2.
      pgevtoam = 1.0735442060000e-09  ! amu / eV.            39 ppb.

c.... Equivalent:  h * L / c.
      pgrmtoam = 1.3310250420000e-15  ! amu / (1/m).         78 ppb.

c.... Equivalent:  h * L / c**2.
      pghztoam = 4.4398216370000e-24  ! amu / Hz.            78 ppb.

c.... Equivalent:  2 * Ry * h * L / c.
      pghttoam = 2.9212623040000e-08  ! amu / hartree.      7.5 ppb.

c....                  Conversion to joule energy units

c.... Equivalent:  c**2.
      pgkgtoj  = 8.9875517873682e+16  ! J    / kg.            0 ppt.

c.... Equivalent:  c**2 / L.
      pgamtoj  = 1.4924177800000e-10  ! J    / amu.          79 ppb.

c.... Equivalent:  kd.
      pgktoj   = 1.3806503000000e-23  ! J    / K.           1.7 ppm.

c.... Equivalent:  k.
      pgevtoj  = 1.6021764620000e-19  ! J    / eV.           39 ppb.

c.... Equivalent:  h * c.
      pgrmtoj  = 1.9864454400000e-25  ! J    / (1/m).        81 ppb.

c.... Equivalent:  h.
      pghztoj  = 6.6260687600000e-34  ! J    / Hz.           78 ppb.

c.... Equivalent:  2 * Ry * h * c.
      pghttoj  = 4.3597438100000e-18  ! J    / hartree.      78 ppb.

c....               Conversion to Kelvin temperature units

c.... Equivalent:  c**2 / kd.
      pgkgtok  = 6.5096510000000e+39  ! K / kg.             1.7 ppm.

c.... Equivalent:  c**2 / (L * kd).
      pgamtok  = 1.0809528000000e+13  ! K / amu.            1.7 ppm.

c.... Equivalent:  1 / kd.
      pgjtok   = 7.2429640000000e+22  ! K / J.              1.7 ppm.

c.... Equivalent:  k / kd.
      pgevtok  = 11604.506            ! K / eV.             1.7 ppm.

c.... Equivalent:  h * c / kd.
      pgrmtok  = 0.014387752          ! K / (1/m).          1.7 ppm.

c.... Equivalent:  h / kd.
      pghztok  = 4.7992374000000e-11  ! K / Hz.             1.7 ppm.

c.... Equivalent:  2 * Ry * h * c / kd.
      pghttok  = 315774.65            ! K / hartree.        1.7 ppm.

c....                  Conversion to electron volt units

c.... Equivalent:  c**2 / k.
      pgkgtoev = 5.6095892100000e+35  ! eV  / kg.            39 ppb.

c.... Equivalent:  c**2 / (L * k).
      pgamtoev = 9.3149401300000e+08  ! eV  / amu.           40 ppb.

c.... Equivalent:  1 / k.
      pgjtoev  = 6.2415097400000e+18  ! eV  / J.             39 ppb.

c.... Equivalent:  kd / k.
      pgktoev  = 8.6173420000000e-05  ! eV  / K.            1.7 ppm.

c.... Equivalent:  h * c / k.
      pgrmtoev = 1.2398418570000e-06  ! eV  / (1/m).         39 ppb.

c.... Equivalent:  h / k.
      pghztoev = 4.1356672700000e-15  ! eV  / Hz.            39 ppb.

c.... Equivalent:  2 * Ry * h * c / k.
      pghttoev = 27.2113834           ! eV  / hartree.       39 ppb.

c....                   Conversion to wavenumber units

c.... Equivalent:  c / h.
      pgkgtorm = 4.5244392900000e+41  ! (1/m)  / kg.         77 ppb.

c.... Equivalent:  c / (L * h).
      pgamtorm = 7.5130066580000e+14  ! (1/m)  / amu.       7.6 ppb.

c.... Equivalent:  1 / (h * c).
      pgjtorm  = 5.0341176200000e+24  ! (1/m)  / J.          77 ppb.

c.... Equivalent:  kd / (h * c).
      pgktorm  = 69.50356             ! (1/m)  / K.         1.7 ppm.

c.... Equivalent:  k / (h * c).
      pgevtorm = 806554.477           ! (1/m)  / eV.         39 ppb.

c.... Equivalent:  1 / c.
      pghztorm = 3.3356409519815e-09  ! (1/m)  / Hz.          0 ppt.


c.... Equivalent:  2 * Ry.
      pghttorm = 21947463.13710       ! (1/m)  / hartree.   7.7 ppt.

c....                    Conversion to frequency units

c.... Equivalent:  c**2 / h.
      pgkgtohz = 1.3563927700000e+50  ! Hz     / kg.         81 ppb.

c.... Equivalent:  c**2 / (L * h).
      pgamtohz = 2.2523427330000e+23  ! Hz     / amu.       7.5 ppb.

c.... Equivalent:  1 / h.
      pgjtohz  = 1.5091905000000e+33  ! Hz     / J.          79 ppb.

c.... Equivalent:  kd / h.
      pgktohz  = 2.0836644000000e+10  ! Hz     / K.         1.7 ppm.

c.... Equivalent:  k / h.
      pgevtohz = 2.4179894910000e+14  ! Hz     / eV.         39 ppb.

c.... Equivalent:  c.
      pgrmtohz = 299792458.0          ! Hz     / (1/m).       exact.

c.... Equivalent:  2 * Ry * c.
      pghttohz = 6.5796839207350e+15  ! Hz     / hartree.   7.6 ppt.

c....                     Conversion to hartree units

c.... Equivalent:  c / (2 * Ry * h).
      pgkgtoht = 2.0614862200000e+34  ! hartree / kg.        79 ppb.

c.... Equivalent:  c / (2 * Ry * L * h).
      pgamtoht = 3.4231777090000e+07  ! hartree / amu.      7.6 ppb.

c.... Equivalent:  1 / (2 * Ry * h * c).
      pgjtoht  = 2.2937127600000e+17  ! hartree / J.         79 ppb.

c.... Equivalent:  kd / (2 * Ry * h * c).
      pgktoht  = 3.1668153000000e-06  ! hartree / K.        1.7 ppm.

c.... Equivalent:  k / (2 * Ry * h * c).
      pgevtoht = 0.0367493260         ! hartree / eV.        38 ppb.

c.... Equivalent:  1 / (2 * Ry).
      pgrmtoht = 4.5563352527500e-08  ! hartree / (1/m).    7.7 ppt.

c.... Equivalent:  1 / (2 * Ry * c).
      pghztoht = 1.5198298460030e-16  ! hartree / Hz.       7.7 ppt.


c....                     MORE RECENT VALUES

c.... Hyperfine splitting of the ground state of Hydrogen.
c....   V. W. Hughes, in Proceedings of the Tenth International Conference
c....   on Atomic Physics, H. Narumi and I Shimamura, Eds.
c....   (North-Holland, Amsterdam, 1987), pp. 1-34.
      pghyfsph = 1.4204057517670e+09  ! Hz.                 0.7 ppt.

c.... Test for input errors.

      nerr = 0
      if (np .le. 0) then
        nerr = 1
        go to 210
      endif

c.... Find the constants.

      do 110 n = 1, np                ! Loop over returned constants.

        if ((ind(n) .ge. 1) .and.
     &      (ind(n) .le. 217)) then
          indx     = ind(n)
          pgcon(n) = pga(indx)
        else
          pgcon(n) = -1.e99
        endif

  110 continue                        ! End of loop over returned constants.
cbugc***DEBUG begins.
cbug 9902 format (/ 'aptconm results:' /
cbug     &  ('  n=',i4,' ind=',i3,' pgcon=',1pe22.14))
cbug      write ( 3, 9902) (n, ind(n), pgcon(n), n = 1, np)
cbugc***DEBUG ends.

  210 return

c.... End of subroutine aptconm.      (+1 line.)
      end
