      subroutine openx (ius, aname, atype, iopt, isize)
 
c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  openx    Updated     1993 March 17 13:10.
c               openx    Originated  1991 March 12 10:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To connect I/O unit ius to the file aname, of type atype,
c               according to option iopt:
c                 -1:  Open if the file exists.  Otherwise, set isize = -1
c                      and return.
c                  0:  Open if the file exists.  Otherwise, create the file.
c                      If that fails, terminate with an error message.
c                  1:  Open if the file exists.  Otherwise, terminate with an
c                      error message.
c
c     Logic:    Entry points:  openx.
c               Called by:  fing, gamcordi, gamsr, gamtrac, mainx, pcordi,
c                 restar, sets12g, tracker, wrtp7, wrtp7n.
c               Calls:  open (system), stop (system).
c
c     Biblio:   FORTLIB-THEORY and FORTLIB-SUBS (Category 9).
c
c     Glossary:
c
c     aname     Input    Name of file to be opened.  A character expression,
c                          which may have meaningless trailing blanks.
c
c     atype     Input    File type:  'text', 'hardcopy' or 'absolute'.
c                          'text':      ASCII input file.
c                          'hardcopy':  ASCII output file
c                            (same as 'text' on UNICOS).
c                          'absolute':  word-addressable, random-access file.
c                            Does not include rd64-type files tartnd, tartppd
c                            and gamdat.
c
c     iopt      Output   Option flag:
c                          -1:  If file exists, open.  If not, set isize * -1
c                               and return.
c                           0:  If file exists, open.  If not, create.
c                           1:  If file exists, open.  If not, terminate.
c
c     isize     In/Out   Size of file (absolute files only).
c                          Returned as -1 if iopt = -1 and the file does not
c                          exist.
c
c     ius       Input    Number of I/O unit to be connnected to file aname.
c
c     Changes:
c
c       ALE     1992 February 18 9:50.  Deleted module createx, since module
c                 openx provides the same function.
c       ALE     1992 April 9 17:10.  Changed argument itype to atype.
c                 Changed logic to CIVIC FORTRAN.
c       ALE     1992 April 15 13:20.  Changed to check to see if a file create
c                 fails.  Changed error return to -1 if iopt = -1 and file does
c                 not exist.  Added option iopt = 2.
c       ALE     1992 June 25 17:40.  Replace "call create" and "call open"
c                 with ANSI STANDARD FORTRAN "open".
c       ALE     1992 June 30 10:20.  Changed to ANSI STANDARD FORTRAN calls for
c                 text files, CBASE calls for absolute files.
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c
 
c.... Input argument specifications.
 
      character*80 aname              ! Name of file to open.
      character*8 atype               ! File type specification.
 
c.... Local variables.
 
      common /copenx/ acrop           ! Word "Opened" or "Cr/Opend".
      character*8 acrop               ! Word "Opened" or "Cr/Opend".
      common /copenx/ astat           ! File status, 'old' or 'unknown'.
      character*8 astat               ! File status, 'old' or 'unknown'.
 
      common /lopenx/ ierr            ! Error flag from system.
      common /lopenx/ rwaccess        ! Read-write-execute status of file.
 
c.... Initialize expected file status.
 
      if (iopt .eq. 0) then           ! Old or new file ok.
        astat = 'unknown'
        acrop = 'Cr/Opend'
      else                            ! File must exist.
        astat = 'old'
        acrop = 'Opened'
      endif
 
c.... Try to open an existing file, or create a file (iopt = 0).
 
      if ((atype .eq. 'text    ') .or. (atype .eq. 'hardcopy')) then
 
        open (unit=ius, iostat=ierr, file=aname, status=astat)
 
      elseif (atype .eq. 'absolute') then
 
cuni    ierr = izopen (ius, aname, isize, rwaccess)
        ierr = 1
 
      else                            ! Error in file type specification.
 
        ierr = 1
 
      endif                           ! Tested atype.
 
 8120 format (/ '  OPENX ERROR opening file ',a,' on I/O unit=',i5,
     &  '.  ierr=',i5)
 
      if (ierr .ne. 0) then           ! File does not exist.
        if (iopt .eq. -1) then        ! Return a warning flag.
          isize = -1
          go to 210
        elseif (iopt .eq. 0) then     ! Create a new file.
          go to 110
        else                          ! Terminate run.
          write (   6, 8120) aname, ius, ierr
          stop 1
        endif
      endif
 
      go to 130
 
c.... Try to create a new file.
 
  110 if ((atype .eq. 'text    ') .or. (atype .eq. 'hardcopy')) then
 
        open (unit=ius, iostat=ierr, file=aname, status='new')
 
      elseif (atype .eq. 'absolute') then
 
cuni    ierr = izcreate (ius, aname, isize)
        ierr = 1
 
      else                            ! Error in file type specification.
 
        ierr = 1
 
      endif                           ! Tested atype.
 
 8110 format (/ '  OPENX ERROR creating file ',a,' on I/O unit=',i5,
     &  '.  ierr=',i5)
 
      if (ierr .ne. 0) then           ! Error opening or creating file.
        write (   6, 8110) aname, ius, ierr
        stop 1
      endif
 
c.... File has been created or opened.
 
  130 continue
 
  210 continue
 
cbugc***DEBUG begins.
cbug 8130 format (2x,a8,1x,a8,' file ',a,' on I/O unit ',i5,'.  Size=',i9)
cbug      write (   6, 8130) acrop, atype, aname, ius, isize
cbugc***DEBUG ends.

      return
 
c.... End of subroutine openx.        (+ 1 line).
      end
